@@ -1,5 +1,4 @@
 import argparse
-import cgi
 import mimetypes
 import os
 import posixpath
@@ -7,14 +6,15 @@ import shutil
 import stat
 import tempfile
 from importlib import import_module
-from urllib.request import urlretrieve
+from urllib.request import build_opener
 
 import django
 from django.conf import settings
 from django.core.management.base import BaseCommand, CommandError
-from django.core.management.utils import handle_extensions
+from django.core.management.utils import handle_extensions, run_formatters
 from django.template import Context, Engine
 from django.utils import archive
+from django.utils.http import parse_header_parameters
 from django.utils.version import get_docs_version
 
 
@@ -80,6 +80,7 @@ class TemplateCommand(BaseCommand):
         )
 
     def handle(self, app_or_project, name, target=None, **options):
+        self.written_files = []
         self.app_or_project = app_or_project
         self.a_or_an = "an" if app_or_project == "app" else "a"
         self.paths_to_remove = []
@@ -200,10 +201,11 @@ class TemplateCommand(BaseCommand):
                 else:
                     shutil.copyfile(old_path, new_path)
 
+                self.written_files.append(new_path)
                 if self.verbosity >= 2:
                     self.stdout.write("Creating %s" % new_path)
                 try:
-                    shutil.copymode(old_path, new_path)
+                    self.apply_umask(old_path, new_path)
                     self.make_writeable(new_path)
                 except OSError:
                     self.stderr.write(
@@ -222,6 +224,8 @@ class TemplateCommand(BaseCommand):
                 else:
                     shutil.rmtree(path_to_remove)
 
+        run_formatters(self.written_files)
+
     def handle_template(self, template, subdir):
         """
         Determine where the app or project templates are.
@@ -305,8 +309,14 @@ class TemplateCommand(BaseCommand):
 
         if self.verbosity >= 2:
             self.stdout.write("Downloading %s" % display_url)
+
+        the_path = os.path.join(tempdir, filename)
+        opener = build_opener()
+        opener.addheaders = [("User-Agent", f"Django/{django.__version__}")]
         try:
-            the_path, info = urlretrieve(url, os.path.join(tempdir, filename))
+            with opener.open(url) as source, open(the_path, "wb") as target:
+                headers = source.info()
+                target.write(source.read())
         except OSError as e:
             raise CommandError(
                 "couldn't download URL %s to %s: %s" % (url, filename, e)
@@ -315,16 +325,16 @@ class TemplateCommand(BaseCommand):
         used_name = the_path.split("/")[-1]
 
         # Trying to get better name from response headers
-        content_disposition = info.get("content-disposition")
+        content_disposition = headers["content-disposition"]
         if content_disposition:
-            _, params = cgi.parse_header(content_disposition)
+            _, params = parse_header_parameters(content_disposition)
             guessed_filename = params.get("filename") or used_name
         else:
             guessed_filename = used_name
 
         # Falling back to content type guessing
         ext = self.splitext(guessed_filename)[1]
-        content_type = info.get("content-type")
+        content_type = headers["content-type"]
         if not ext and content_type:
             ext = mimetypes.guess_extension(content_type)
             if ext:
@@ -375,6 +385,12 @@ class TemplateCommand(BaseCommand):
         scheme = template.split(":", 1)[0].lower()
         return scheme in self.url_schemes
 
+    def apply_umask(self, old_path, new_path):
+        current_umask = os.umask(0)
+        os.umask(current_umask)
+        current_mode = stat.S_IMODE(os.stat(old_path).st_mode)
+        os.chmod(new_path, current_mode & ~current_umask)
+
     def make_writeable(self, filename):
         """
         Make sure that the file is writeable.
