@@ -17,6 +17,8 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
     sql_create_inline_fk = (
         "REFERENCES %(to_table)s (%(to_column)s) DEFERRABLE INITIALLY DEFERRED"
     )
+    sql_create_column_inline_fk = sql_create_inline_fk
+    sql_delete_column = "ALTER TABLE %(table)s DROP COLUMN %(column)s"
     sql_create_unique = "CREATE UNIQUE INDEX %(name)s ON %(table)s (%(columns)s)"
     sql_delete_unique = "DROP INDEX %(name)s"
 
@@ -69,6 +71,9 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                 "Cannot quote parameter value %r of type %s" % (value, type(value))
             )
 
+    def prepare_default(self, value):
+        return self.quote_value(value)
+
     def _is_referenced_by_fk_constraint(
         self, table_name, column_name=None, ignore_self=False
     ):
@@ -82,13 +87,10 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             for other_table in self.connection.introspection.get_table_list(cursor):
                 if ignore_self and other_table.name == table_name:
                     continue
-                constraints = (
-                    self.connection.introspection._get_foreign_key_constraints(
-                        cursor, other_table.name
-                    )
+                relations = self.connection.introspection.get_relations(
+                    cursor, other_table.name
                 )
-                for constraint in constraints.values():
-                    constraint_table, constraint_column = constraint["foreign_key"]
+                for constraint_column, constraint_table in relations.values():
                     if constraint_table == table_name and (
                         column_name is None or constraint_column == column_name
                     ):
@@ -215,7 +217,12 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             alter_field and getattr(alter_field[1], "primary_key", False)
         ):
             for name, field in list(body.items()):
-                if field.primary_key:
+                if field.primary_key and not (
+                    # Do not remove the old primary key when an altered field
+                    # that introduces a primary key is the same field.
+                    alter_field
+                    and name == alter_field[1].name
+                ):
                     field.primary_key = False
                     restore_pk_field = field
                     if field.auto_created:
@@ -226,8 +233,8 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             body[create_field.name] = create_field
             # Choose a default and insert it into the copy map
             if not create_field.many_to_many and create_field.concrete:
-                mapping[create_field.column] = self.quote_value(
-                    self.effective_default(create_field)
+                mapping[create_field.column] = self.prepare_default(
+                    self.effective_default(create_field),
                 )
         # Add in any altered fields
         if alter_field:
@@ -238,7 +245,7 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             if old_field.null and not new_field.null:
                 case_sql = "coalesce(%(col)s, %(default)s)" % {
                     "col": self.quote_name(old_field.column),
-                    "default": self.quote_value(self.effective_default(new_field)),
+                    "default": self.prepare_default(self.effective_default(new_field)),
                 }
                 mapping[new_field.column] = case_sql
             else:
@@ -371,14 +378,22 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
                     self.deferred_sql.remove(sql)
 
     def add_field(self, model, field):
-        """
-        Create a field on a model. Usually involves adding a column, but may
-        involve adding a table instead (for M2M fields).
-        """
-        # Special-case implicit M2M tables
-        if field.many_to_many and field.remote_field.through._meta.auto_created:
-            return self.create_model(field.remote_field.through)
-        self._remake_table(model, create_field=field)
+        """Create a field on a model."""
+        if (
+            # Primary keys and unique fields are not supported in ALTER TABLE
+            # ADD COLUMN.
+            field.primary_key
+            or field.unique
+            or
+            # Fields with default values cannot by handled by ALTER TABLE ADD
+            # COLUMN statement because DROP DEFAULT is not supported in
+            # ALTER TABLE.
+            not field.null
+            or self.effective_default(field) is not None
+        ):
+            self._remake_table(model, create_field=field)
+        else:
+            super().add_field(model, field)
 
     def remove_field(self, model, field):
         """
@@ -391,6 +406,16 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
             if field.remote_field.through._meta.auto_created:
                 self.delete_model(field.remote_field.through)
             # For explicit "through" M2M fields, do nothing
+        elif (
+            self.connection.features.can_alter_table_drop_column
+            # Primary keys, unique fields, indexed fields, and foreign keys are
+            # not supported in ALTER TABLE DROP COLUMN.
+            and not field.primary_key
+            and not field.unique
+            and not field.db_index
+            and not (field.remote_field and field.db_constraint)
+        ):
+            super().remove_field(model, field)
         # For everything else, remake.
         else:
             # It might not actually have a column behind it
@@ -431,7 +456,11 @@ class DatabaseSchemaEditor(BaseDatabaseSchemaEditor):
         # Alter by remaking table
         self._remake_table(model, alter_field=(old_field, new_field))
         # Rebuild tables with FKs pointing to this field.
-        if new_field.unique and old_type != new_type:
+        old_collation = old_db_params.get("collation")
+        new_collation = new_db_params.get("collation")
+        if new_field.unique and (
+            old_type != new_type or old_collation != new_collation
+        ):
             related_models = set()
             opts = new_field.model._meta
             for remote_field in opts.related_objects:
